@@ -31,10 +31,11 @@ finbif_informal_groups <- function(group, limit = 50, quiet = FALSE) {
   if (!quiet) {
     cat(df[["tree"]][seq_len(limit)], sep = "\n")
     extra <- n - limit
-    if (extra > 0L)
+    if (extra > 0L) {
       cat(
         "...", extra, " more group", if (extra == 1) "" else "s", "\n", sep = ""
       )
+    }
   }
   invisible(df[["name"]])
 }
